@@ -5,7 +5,7 @@
  | program/include/html.php                                              |
  |                                                                       |
  | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2005-2008, RoundCube Dev, - Switzerland                 |
+ | Copyright (C) 2005-2010, RoundCube Dev, - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: $
+ $Id: html.php 3367 2010-03-17 12:24:09Z thomasb $
 
  */
 
@@ -32,9 +32,9 @@ class html
     protected $allowed = array();
     protected $content;
 
-    public static $common_attrib = array('id','class','style','title','align');
-    public static $containers = array('iframe','div','span','p','h1','h2','h3','form','textarea','table','tr','th','td','style');
     public static $lc_tags = true;
+    public static $common_attrib = array('id','class','style','title','align');
+    public static $containers = array('iframe','div','span','p','h1','h2','h3','form','textarea','table','thead','tbody','tr','th','td','style','script');
 
     /**
      * Constructor
@@ -129,7 +129,8 @@ class html
         if (is_string($attr)) {
             $attr = array('src' => $attr);
         }
-        return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib, array('src','alt','width','height','border','usemap')));
+        return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib,
+	    array('src','alt','width','height','border','usemap')));
     }
 
     /**
@@ -145,7 +146,8 @@ class html
         if (is_string($attr)) {
             $attr = array('href' => $attr);
         }
-        return self::tag('a', $attr, $cont, array_merge(self::$common_attrib, array('href','target','name','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
+        return self::tag('a', $attr, $cont, array_merge(self::$common_attrib,
+	    array('href','target','name','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
     }
 
     /**
@@ -192,7 +194,8 @@ class html
         if (is_string($attr)) {
             $attr = array('src' => $attr);
         }
-        return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib, array('src','name','width','height','border','frameborder')));
+        return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib,
+	    array('src','name','width','height','border','frameborder')));
     }
 
     /**
@@ -263,7 +266,9 @@ class html_inputfield extends html
 {
     protected $tagname = 'input';
     protected $type = 'text';
-    protected $allowed = array('type','name','value','size','tabindex','autocomplete','checked','onchange','onclick','disabled','readonly','spellcheck','results');
+    protected $allowed = array('type','name','value','size','tabindex',
+	'autocomplete','checked','onchange','onclick','disabled','readonly',
+	'spellcheck','results','maxlength','src');
 
     public function __construct($attrib = array())
     {
@@ -431,7 +436,8 @@ class html_checkbox extends html_inputfield
 class html_textarea extends html
 {
     protected $tagname = 'textarea';
-    protected $allowed = array('name','rows','cols','wrap','tabindex','onchange','disabled','readonly','spellcheck');
+    protected $allowed = array('name','rows','cols','wrap','tabindex',
+	'onchange','disabled','readonly','spellcheck');
 
     /**
      * Get HTML code for this object
@@ -457,11 +463,12 @@ class html_textarea extends html
             unset($this->attrib['value']);
         }
 
-        if (!empty($value) && !ereg('mce_editor', $this->attrib['class'])) {
+        if (!empty($value) && !preg_match('/mce_editor/', $this->attrib['class'])) {
             $value = Q($value, 'strict', false);
         }
 
-        return self::tag($this->tagname, $this->attrib, $value, array_merge(self::$common_attrib, $this->allowed));
+        return self::tag($this->tagname, $this->attrib, $value,
+	    array_merge(self::$common_attrib, $this->allowed));
     }
 }
 
@@ -488,7 +495,8 @@ class html_select extends html
 {
     protected $tagname = 'select';
     protected $options = array();
-    protected $allowed = array('name','size','tabindex','autocomplete','multiple','onchange','disabled');
+    protected $allowed = array('name','size','tabindex','autocomplete',
+	'multiple','onchange','disabled');
     
     /**
      * Add a new option to this drop-down
@@ -546,7 +554,9 @@ class html_select extends html
 class html_table extends html
 {
     protected $tagname = 'table';
-    protected $allowed = array('id','class','style','width','summary','cellpadding','cellspacing','border');
+    protected $allowed = array('id','class','style','width','summary',
+	'cellpadding','cellspacing','border');
+
     private $header = array();
     private $rows = array();
     private $rowindex = 0;
@@ -591,7 +601,7 @@ class html_table extends html
     public function add_header($attr, $cont)
     {
         if (is_string($attr))
-        $attr = array('class' => $attr);
+    	    $attr = array('class' => $attr);
 
         $cell = new stdClass;
         $cell->attrib = $attr;
@@ -599,6 +609,34 @@ class html_table extends html
         $this->header[] = $cell;
     }
 
+     /**
+     * Remove a column from a table
+     * Useful for plugins making alterations
+     * 
+     * @param string $class 
+     */
+    public function remove_column($class)
+    {
+        // Remove the header
+        foreach($this->header as $index=>$header){
+            if($header->attrib['class'] == $class){
+                unset($this->header[$index]);
+                break;
+            }
+        }
+
+        // Remove cells from rows
+        foreach($this->rows as $i=>$row){
+            foreach($row->cells as $j=>$cell){
+                if($cell->attrib['class'] == $class){
+                    unset($this->rows[$i]->cells[$j]);
+                    break;
+                }
+            }
+        }
+    }
+
+
     /**
      * Jump to next row
      *
@@ -613,6 +651,18 @@ class html_table extends html
         $this->rows[$this->rowindex]->cells = array();
     }
 
+    /**
+     * Set current row attrib
+     *
+     * @param array Row attributes
+     */
+    public function set_row_attribs($attr = array())
+    {
+        if (is_string($attr))
+    	    $attr = array('class' => $attr);
+
+        $this->rows[$this->rowindex]->attrib = $attr;
+    }
 
     /**
      * Build HTML output of the table data
@@ -657,6 +707,16 @@ class html_table extends html
         unset($this->attrib['cols'], $this->attrib['rowsonly']);
         return parent::show();
     }
+    
+    /**
+     * Count number of rows
+     *
+     * @return The number of rows
+     */
+    public function size()
+    {
+      return count($this->rows);
+    }
 }
 
 ?>
